Package: IsoCorrectoR
Title: Correction for natural isotope abundance and tracer purity in MS
        and MS/MS data from stable isotope labeling experiments
Version: 1.29.0
Authors@R: c(person("Christian","Kohler",email="christian.kohler@ur.de",role=c("cre","aut")),
  person("Paul","Heinrich",email="paul.heinrich@ur.de",role=c("aut")))
Imports: dplyr, magrittr, methods, quadprog, readr, readxl, stringr,
        tibble, tools, utils, pracma, WriteXLS
Description: IsoCorrectoR performs the correction of mass spectrometry data from stable isotope labeling/tracing metabolomics experiments with regard to natural isotope abundance and tracer impurity. Data from both MS and MS/MS measurements can be corrected (with any tracer isotope: 13C, 15N, 18O...), as well as ultra-high resolution MS data from multiple-tracer experiments (e.g. 13C and 15N used simultaneously). See the Bioconductor package IsoCorrectoRGUI for a graphical user interface to IsoCorrectoR. NOTE: With R version 4.0.0, writing correction results to Excel files may currently not work on Windows. However, writing results to csv works as before.
Depends: R (>= 3.5)
URL: https://genomics.ur.de/files/IsoCorrectoR/
License: GPL-3
LazyData: TRUE
NeedsCompilation: no
biocViews: Software, Metabolomics, MassSpectrometry, Preprocessing,
        ImmunoOncology
RoxygenNote: 6.1.1
Suggests: IsoCorrectoRGUI, knitr, rmarkdown, testthat, BiocStyle
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/IsoCorrectoR
git_branch: devel
git_last_commit: 9553234
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
Packaged: 2025-10-30 21:39:19 UTC; biocbuild
Author: Christian Kohler [cre, aut],
  Paul Heinrich [aut]
Maintainer: Christian Kohler <christian.kohler@ur.de>
