# Generated by roxygen2: do not edit by hand

export(IsoCorrection)
import(quadprog)
import(utils)
importFrom(WriteXLS,WriteXLS)
importFrom(dplyr,arrange_)
importFrom(dplyr,last)
importFrom(magrittr,'%>%')
importFrom(methods,is)
importFrom(pracma,lsqlincon)
importFrom(readr,read_csv)
importFrom(readxl,read_excel)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_replace)
importFrom(stringr,str_split)
importFrom(tibble,as_tibble)
importFrom(tools,file_ext)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,write.csv)
