#' IsoCorrectoR example data
#' 
#' Example data for the IsoCorrectoR package
#' 
#' @format A list with four elements, namely
#' \describe{
#' \item{tool_features: }{overview of correction features as provided by various tools IsoCorrectoR is compared with.}
#' \item{element_file: }{example data for an Element file. This information is independent of the resolution type (normal or high)}
#' \item{normal_resolution: }{list with two elements, containing examples for normal resolution data: molecule information and measurement data}
#' \item{high_resolution: }{list with two elements, containing examples for high resolution data: molecule information and measurement data}
#' }
#' 
#' 
#' @usage 
#' data(IsoCorrectoR)
#' 
#' @author Paul Heinrich, Christian Kohler
"IsoCorrectoR"
